@@ -359,6 +359,10 @@ int main( int argc, char *argv[] ) {
    * components easily. 
    */
   context=context_new(old_context);
+  if(context == 0) {
+    fprintf(stderr,_("failed to get new context.\n"));
+    exit(-1);
+  }
 
   freecon(old_context);
   /* Make `pw' point to a structure containing the data              *
