@@ -51,6 +51,7 @@
 #include <ctype.h>
 #include <libintl.h>
 #include <locale.h>
+#include <limits.h>
 #define _(msgid) gettext (msgid)
 #ifndef PACKAGE
 #define PACKAGE "policycoreutils"   /* the name of this package lang translation */
@@ -60,7 +61,7 @@
   where: <script> is the name of the init script to run,\n\
          <args ...> are the arguments to that script.")
 
-#define CONTEXT_FILE "/etc/security/initrc_context"
+#define CONTEXT_FILE "initrc_context"
 #ifdef USE_PAM
 
 /************************************************************************
@@ -269,10 +270,11 @@ int get_init_context(security_context_t *context) {
   FILE* fp;
   char  buf[255], *bufp;
   int buf_len;
-  
-  fp = fopen(CONTEXT_FILE, "r");
+  char context_file[PATH_MAX];
+  snprintf(context_file, sizeof(context_file)-1, "%s/%s", selinux_contexts_path(), CONTEXT_FILE);
+  fp = fopen(context_file, "r");
   if (!fp) {
-    fprintf(stderr, _("Could not open file %s\n"), CONTEXT_FILE);
+    fprintf(stderr, _("Could not open file %s\n"), context_file);
     return -1;
   }
 
@@ -299,7 +301,7 @@ int get_init_context(security_context_t *context) {
   }
 out:
   fclose(fp);
-  fprintf(stderr, _("No context in file %s\n"), CONTEXT_FILE);
+  fprintf(stderr, _("No context in file %s\n"), context_file);
   return -1;
 
 } /* get_init_context() */
