@@ -2,6 +2,7 @@
  * Copyright 1999-2004 Gentoo Technologies, Inc.
  * Distributed under the terms of the GNU General Public License v2
  * $Header: /home/cvsroot/gentoo-projects/hardened/policycoreutils-extra/src/sestatus.c,v 1.10 2004/03/26 19:25:52 pebenito Exp $
+ * Patch provided by Steve Grubb
  */
 
 #include <stdio.h>
@@ -9,6 +10,7 @@
 #include <string.h>
 #include <errno.h>
 #include <selinux/selinux.h>
+#include <selinux/get_default_type.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -49,7 +51,7 @@ int cmp_cmdline(const char *command, int pid) {
 		buf[BUFSIZE-1] = '\0';
 
 	/* check if this is the command we're looking for. */
-	if( strncmp(command,buf,strlen(command)) == 0 )
+	if (strcmp(command,buf) == 0)
 		return 1;
 	else
 		return 0;
@@ -67,7 +69,9 @@ int pidof(const char *command) {
 	}
 
 	while( (de = readdir(dir)) != NULL) {
-		if (!(pid = atoi (de->d_name)) || pid == self)
+		errno = 0;
+		pid = (int)strtol(de->d_name, (char **)NULL, 10);
+		if ( errno || pid == 0 || pid == self)
 			continue;
 		if(cmp_cmdline(command,pid)) {
 			ret=pid;
@@ -101,8 +105,10 @@ void load_checks(char *pc[], int *npc, char *fc[], int *nfc) {
 			buf[buf_len-1] = 0;
 
 		bufp = buf;
-		while(*bufp && isspace(*bufp))
+		while(*bufp && isspace(*bufp)) {
 			bufp++;
+			buf_len--;
+		}
 
 		if(*bufp == '#')
 			/* skip comments */
@@ -177,6 +183,10 @@ int main(int argc, char **argv) {
 	char **bools;
 	int nbool;
 
+	/* policy */
+	const char *pol_name;
+	char *pol_path;
+
 	opt=getopt(argc, argv, "v");
 	if(opt == '?') {
 		/* invalid option */
@@ -226,6 +236,23 @@ int main(int argc, char **argv) {
 			break;
 	}
 
+	printf_tab("Mode from config file:");
+	if (selinux_getenforcemode(&rc)==0) {
+		switch (rc) {
+		case 1:
+			printf("enforcing\n");
+			break;
+		case 0:
+			printf("permissive\n");
+			break;
+		case -1:
+			printf("disabled\n");
+			break;
+		}
+	} else {
+		printf("error (%s)\n", strerror(errno));
+	}
+
 	rc=security_policyvers();
 	printf_tab("Policy version:");
 	if(rc < 0)
@@ -233,6 +260,17 @@ int main(int argc, char **argv) {
 	else
 		printf("%d\n",rc);
 
+	/* Dump all the path information */
+	printf_tab("Policy from config file:");
+	pol_path = strdup(selinux_policy_root());
+	if (pol_path) {
+		pol_name = basename(pol_path);
+		puts(pol_name);
+		free(pol_path);
+	} else {
+		printf("error (%s)\n", strerror(errno));
+	}
+
 	/* show booleans */
 	if(security_get_boolean_names(&bools,&nbool) >= 0) {
 		printf("\nPolicy booleans:\n");
