@@ -12,15 +12,19 @@
  * the user.  The program does not cross file system boundaries.
  *
  * USAGE:
- * setfiles [-dnpqsvW] spec_file pathname...
+ * setfiles [-dnpqsvW] [-e directory ] [-c policy] [-o filename ] spec_file pathname...
  * 
+ * -e   Specify directory to exclude
+ * -c   Verify the specification file using a binary policy
  * -d   Show what specification matched each file.
+ * -l   Log changes in files labels to syslog.
  * -n	Do not change any file labels.
  * -q   Be quiet (suppress non-error output).
  * -r   Use an alternate root path
  * -s   Use stdin for a list of files instead of searching a partition.
  * -v	Show changes in file labels.  
  * -W   Warn about entries that have no matching file.
+ * -o filename write out file names with wrong context.
  *
  * spec_file	The specification file.
  * pathname...	The file systems to label (omit if using -s).	
@@ -65,19 +69,33 @@
 #define __USE_XOPEN_EXTENDED 1	/* nftw */
 #include <ftw.h>
 #include <limits.h>
+#include <sepol/sepol.h>
 #include <selinux/selinux.h>
+#include <syslog.h> 
+#include <libgen.h>
 
 static int add_assoc = 1;
+static FILE *outfile=NULL;
+
+#define MAX_EXCLUDES 100
+static int excludeCtr=0;
+struct edir {
+	char *directory;
+        int size;
+};
+static struct edir excludeArray[MAX_EXCLUDES];
 
 /*
  * Command-line options.
  */
+static char *policyfile = NULL;
 static int debug = 0;
 static int change = 1;
 static int quiet = 0;
 #define QPRINTF(args...) do { if (!quiet) printf(args); } while (0)
 static int use_stdin = 0;
 static int verbose = 0;
+static int log = 0;
 static int warn_no_match = 0;
 static char *rootpath = NULL;
 static int rootpathlen = 0;
@@ -135,6 +153,18 @@ int get_stem_from_spec(const char * const buf)
 	return tmp - buf;
 }
 
+static int exclude(const char *file) {
+	int i=0;
+	for(i=0; i < excludeCtr; i++) { 
+		if (strncmp(file,excludeArray[i].directory,excludeArray[i].size)==0) {
+			if (file[excludeArray[i].size]==0 || 
+			    file[excludeArray[i].size]=='/') {
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
 /* return the length of the text that is the stem of a file name */
 int get_stem_from_file_name(const char * const buf)
 {
@@ -388,6 +418,11 @@ int match(const char *name, struct stat *sb)
 		buf += rootpathlen;
 	}
 
+	if (excludeCtr > 0) {
+		if (exclude(fullname)) {
+			return -1;
+		}
+	}
 	ret = lstat(fullname, sb);
 	if (ret) {
 		fprintf(stderr, "%s:  unable to stat file %s\n", progname,
@@ -480,8 +515,9 @@ int nodups_specs()
 void usage(const char * const name)
 {
 	fprintf(stderr,
-		"usage:  %s [-dnqvW] spec_file pathname...\n"
-		"usage:  %s -s [-dnqvW] spec_file\n", name, name);
+		"usage:  %s [-dnqvW] [-o filename] spec_file pathname...\n"
+		"usage:  %s [-c policyfile] spec_file\n"
+		"usage:  %s -s [-dnqvW] [-o filename ] spec_file\n", name, name, name);
 	exit(1);
 }
 
@@ -650,6 +686,16 @@ static int apply_spec(const char *file,
 		}
 	}
 
+	if (log &&
+	    !only_changed_user(context, spec_arr[i].context)) {
+		syslog(LOG_INFO, "relabeling %s from %s to %s\n", 
+		       my_file, context, spec_arr[i].context);
+	}
+
+	if (outfile &&
+	    !only_changed_user(context, spec_arr[i].context)) 
+		fprintf(outfile, "%s\n", my_file);
+
 	freecon(context);
 
 	/*
@@ -690,6 +736,28 @@ void set_rootpath(const char *arg)
 	rootpathlen = len;
 }
 
+void validate_context(const char *context, const char *progname,
+		      int lineno)
+{
+	int valid = 1;
+
+	if (policyfile) {
+		valid = (sepol_check_context ((char *) context) >= 0);
+	} else if (security_check_context((char *) context) < 0 && errno != ENOENT) {
+		valid = 0;
+		inc_err();
+	}
+
+	if (!valid) {
+		fprintf(stderr, "%s:  invalid context %s on line number %d\n",
+			progname, context, lineno);
+
+		/* Exit immediately if we're in checking mode. */
+		if (policyfile)
+			exit(1);
+	}
+}
+
 int main(int argc, char **argv)
 {
 	FILE *fp;
@@ -704,15 +772,83 @@ int main(int argc, char **argv)
 	int opt, items, len, lineno, pass, regerr, i, j;
 	spec_t *spec_copy;
 
+	memset(excludeArray,0, sizeof(excludeArray));
+
 	/* Process any options. */
-	while ((opt = getopt(argc, argv, "dnqrsvW")) > 0) {
+	while ((opt = getopt(argc, argv, "c:dlnqrsvWe:o:")) > 0) {
 		switch (opt) {
+		case 'c':
+		{
+			FILE *policystream;
+			
+			policyfile = optarg;
+
+			policystream = fopen(policyfile,"r");
+			if (!policystream) {
+				fprintf(stderr, "Error opening %s: %s\n", 
+					policyfile, strerror(errno));
+				exit(1);
+			}
+
+			if (sepol_set_policydb_from_file(policystream) < 0) {
+				fprintf(stderr, "Error reading policy %s: %s\n", policyfile,
+					strerror(errno));
+				exit(1);
+			}
+			fclose(policystream);
+			
+			break;
+		}
+		case 'e':
+		{
+			int len;
+			struct stat sb;
+			if(optarg[0] != '/') {
+				fprintf(stderr, "Full path required for exclude: %s.\n", 
+					optarg);
+				exit(1);
+			}
+			if(lstat(optarg, &sb)) {
+				fprintf(stderr, "Directory \"%s\" not found.\n", optarg);
+				exit(1);
+			}
+			if ((sb.st_mode & S_IFDIR) == 0 ) {
+				fprintf(stderr, "\"%s\" is not a Directory: mode %o\n", optarg,sb.st_mode);
+				exit(1);
+			}
+			len=strlen(optarg);
+			excludeArray[excludeCtr].directory = strdup(optarg);
+			if (!excludeArray[excludeCtr].directory) {
+				fprintf(stderr, "Out of memory.\n");
+				exit(1);
+			}
+			excludeArray[excludeCtr++].size = len;
+			if (excludeCtr > MAX_EXCLUDES) {
+				fprintf(stderr, "Maximum excludes %d exceeded.\n", 
+					MAX_EXCLUDES);
+				exit(1);
+			}
+			break;
+		}
+			
 		case 'd':
 			debug = 1;
 			break;
+		case 'l':
+			log = 1;
+			break;
 		case 'n':
 			change = 0;
 			break;
+		case 'o':
+			outfile = fopen(optarg,"w");
+			if (!outfile) {
+				fprintf(stderr, "Error opening %s: %s\n", 
+					optarg, strerror(errno));
+
+				usage(argv[0]);
+			}
+			break;
 		case 'q':
 			quiet = 1;
 			break;
@@ -745,7 +881,10 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (use_stdin) {
+	if (policyfile) {
+		if (optind != (argc - 1))
+			usage(argv[0]);
+	} else if (use_stdin) {
 		if (optind != (argc - 1)) {
 			/* Cannot mix with pathname arguments. */
 			usage(argv[0]);
@@ -889,13 +1028,7 @@ int main(int argc, char **argv)
 				spec_arr[nspec].context = context;
 
 				if (strcmp(context, "<<none>>")) {
-					if (security_check_context(context) < 0 && errno != ENOENT) {
-						fprintf(stderr,
-							"%s:  invalid context %s on line number %d\n",
-							argv[0], context,
-							lineno);
-						inc_err();
-					}
+					validate_context(context, argv[0], lineno);
 				}
 
 				/* Determine if specification has 
@@ -971,7 +1104,7 @@ int main(int argc, char **argv)
 			strtok(buf, "\n");
 			if(buf[0] != '\n')
 			{
-				if(stat(buf, &sb))
+				if(lstat(buf, &sb))
 					fprintf(stderr, "File \"%s\" not found.\n", buf);
 				else
 				{
@@ -1039,6 +1172,12 @@ int main(int argc, char **argv)
 			}
 		}
 	}
+	if (outfile) 
+		fclose(outfile);
+
+	for(i=0; i < excludeCtr; i++) { 
+		free(excludeArray[i].directory);
+	}
 
 	QPRINTF("%s:  Done.\n", argv[0]);
 
